<?php
  $courselist = _moodle_get_courses('today', '+60 days');   // todo

  if (empty($courselist)) {
    return;     // nothing to do
  }
  /* DEBUG */
  echo "<pre>";
  /* END DEBUG */

  // initialize a list of course ids which need to be considered.
  $cids = array_combine(array_keys($courselist), array_keys($courselist) );

  // process each node of the moodle course type.
  $result = db_query("SELECT n.nid FROM {node} n WHERE n.type = 'course' AND n.status = 1");

  while ($node = db_fetch_object($result)) {
    $node = node_load($node->nid);    // Drupal course node
    $node_cid = $node->field_courseid[0]['value'];  // this node's course id
    // course id stored in DB as an integer, but returned as a string.  wtf.
    $node_cid = (int)$node_cid;

    /* DEBUG */
    echo "\n --- CHECKING node, nid = {$node->nid}, courseid = $node_cid\n";
    /* END DEBUG */

    if ($node_cid < 2) {
      watchdog('moodle', 'Node nid %nid has bogus course id %cid', array('%nid' => $node->nid, '%cid' => $node_cid), WATCHDOG_ERROR);
      continue;
    }

    if (!in_array($node_cid, $cids)) {

      /* DEBUG */
      echo "course id $node_cid not returned from moodle -- UNPUBLISHING node\n";
      /* END DEBUG */

      // course ID no longer returned from Moodle, assume inactive/deleted
      $node->status = 0;    // unpublish
      node_save($node);
      watchdog('moodle', '@type: unpublished %title. Course id %cid no longer exists in Moodle.', array('@type' => $node->type, '%title' => $node->title, '%cid' => $node_cid), WATCHDOG_NOTICE, l(t('view'), 'node/' . $node->nid));
      continue;
    }

    /* DEBUG */
    echo "removing course id $node_cid from list\n";
    /* END DEBUG */

    // as we process each course id, remove it from the list
    $cids[$node_cid] = NULL;
    unset($cids[$node_cid]);

    $course = $courselist[$node->field_courseid[0]['value']];
    // check for any recent changes to existing courses
    // this doesn't work
    //if ($node->changed >= $course['timemodified']) {
    if (0) {

      /* DEBUG */
      echo "node is more recent than moodle course -- SKIPPING\n";
      echo "  node timestamp: {$node->changed}\n";
      echo "course timestamp: {$course['timemodified']}\n";
      /* END DEBUG */

      continue; // Drupal node is more recent than Moodle course
    }

    // update Drupal node
    $instructors = _moodle_get_instructors($node_cid);
/* DEBUG */
print "updating instructors: ";
var_dump($instructors);
/* END DEBUG */
    if ($instructors === FALSE) $instructors = array('unknown');
    moodle_set_node($course, $instructors, $node);
    node_save($node);
    watchdog('moodle', '@type: updated %title.', array('@type' => $node->type, '%title' => $node->title), WATCHDOG_NOTICE, l(t('view'), 'node/' . $node->nid));
  }

  // any course ids left in $cids are those which don't have matching nodes.
  // create new course nodes for them.
  foreach ($cids as $cid) {

/* DEBUG */
echo "attempting to create node for course # $cid\n";
/* END DEBUG */

    unset($node);
    $node = new stdClass();
    $node->type = 'course';
    $node->field_courseid[0]['value'] = $cid;

    // we set node directly instead of using drupal_execute() due to CCK bug.
    $instructors = _moodle_get_instructors($cid);
    if ($instructors === FALSE) $instructors = array('unknown');
/* DEBUG */
print "inserting instructors: ";
print_r($instructors);
/* END DEBUG */
    moodle_set_node($courselist[$cid], $instructors, $node);
    $node = node_submit($node);
    node_save($node);
    watchdog('moodle', '@type: added %title.', array('@type' => $node->type, '%title' => $node->title), WATCHDOG_NOTICE, l(t('view'), 'node/' . $node->nid));
  }
  variable_set('moodle_last_update', time());

function moodle_set_node($course, $instructors, &$node) {
  // get and convert starting date to unixtime, calculate ending date.
  // assumes timezone of drupal and moodle are same!  todo  fixme
  list($year, $month, $day) = explode('-', $course['startdate']);
  $startdate = mktime(0, 0, 0, $month, $day, $year);
  $enddate = $startdate + ((3600*24*7) * $course['numweeks']);

  $node->title                = $course['fullname'];
  $node->changed              = $course['timemodified'];

  $node->field_body[0]['value']     = $course['summary'];
  $node->field_isevent[0]['value']  = '1';

  $node->field_date[0]['value']     = $startdate;
  $node->field_date[0]['value2']    = $enddate;

  $node->field_url[0]['value']      = _moodle_get_url() . substr($course['urlpath'], 1);  // skip leading slash

  for ($i = 0; $i < count($instructors); $i++) {
    $node->field_name_of_instructors[$i]['value'] = $instructors[$i];
  }
}
?>
